#!/bin/bash
#
# buildJsoncpp.sh 
# (in source build required)
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e10|e14|e15> <prof|debug> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}:${PRODUCTS}
}

function setup_and_verify()
{
    # this should not complain
    echo "Finished building ${package} ${pkgver}"
    setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
    echo "${package} is installed at ${JSONCPP_FQ_DIR}"
}


# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=jsoncpp
origpkgver=v1_7_7
pkgver=${origpkgver}a
ssibuildshims_version=v1_03_02
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' -e 's/^v//'`
pkgtar=jsoncpp-${pkgdotver}.tar.gz

make_tarball_opts=("\${product_dir}" "\${package}" "\${pkgver}" "\${fullqual}")

get_this_dir
get_ssibuildshims
source define_basics

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   eval ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}"
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
debug) extra_command="-DCMAKE_BUILD_TYPE=Debug";;
prof)  extra_command="-DCMAKE_BUILD_TYPE="RelWithDebug;;
* )
   echo "ERROR: please specify debug or prof"
   usage
   exit 1
;;
esac

if [[ "${basequal}" == e1[0245] ]]
then
  cxxflg="-std=c++14"
elif [[ "${basequal}" == e13 ]]
then
  cxxflg="-std=c++14"
else
  ssi_die "Qualifier $basequal not recognized."
fi
extra_command="${extra_command} -DCMAKE_CXX_FLAGS=${cxxflg}"

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

# We need cmake
source setup_cmake || ssi_die "ERROR: failed to setup camke"


# jsoncpp's makefiles can not do an out-of-source build.
cd ${pkgdir} || ssi_die "Unable to cd to ${pkgdir}"
tar xf ${tardir}/${pkgtar} || ssi_die "Unable to unwind ${tardir}/${pkgtar} into ${PWD}"

# We only make an optimized build, because jsoncpp does not support
# anything else (and nobody would really want it anyway).
cd ${pkgdir}/jsoncpp-${pkgdotver} || ssi_die "Unable to cd to ${pkgdir}/jsoncpp-${pkgdotver}"

set -x

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

FC=gfortran \
CC=gcc \
CXX=g++ \
    cmake ${extra_command} \
    -DCMAKE_INSTALL_PREFIX=${pkgdir} \
    -DBUILD_SHARED_LIBS=ON \
    -DBUILD_STATIC_LIBS=OFF \
    ${pkgdir}/jsoncpp-${pkgdotver} || ssi_die "Failed to run CMake"
make -j $ncore || ssi_die "Failed in 'make'"
make install || ssi_die "Failed to install"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

setup_and_verify

# fix embedded path
jsonpkg=${JSONCPP_LIB}/pkgconfig/jsoncpp.pc
mv ${jsonpkg} ${jsonpkg}.bak
cat ${jsonpkg}.bak | sed -e 's%^prefix=.*$%prefix=${pcfiledir}/../..%' > ${jsonpkg}

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   eval ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}"
fi

exit 0
